from __future__ import annotations

import ctypes
import sys
from os import PathLike


def is_admin() -> bool:
    try:
        return bool(ctypes.windll.shell32.IsUserAnAdmin())
    except WindowsError:
        return False


def runas(fp: str | PathLike[str]) -> int:
    """以管理员身份运行.
    返回42为成功, 5为失败
    """
    return ctypes.windll.shell32.ShellExecuteW(None, "runas", sys.executable, fp, None, 1)



from __future__ import annotations

import os
import winreg


class Key:
    def __init__(self, key: int, sub_key: str) -> None:
        self.key = key
        self.sub_key = sub_key

    def open(self, reserved: int = 0, access: int = winreg.KEY_READ):
        return winreg.OpenKey(self.key, self.sub_key, reserved, access)


class HKEY_CLASSES_ROOT:
    HKEY = winreg.HKEY_CLASSES_ROOT
    FILE_MENU = Key(HKEY, r"*\shell")


class HKEY_LOCAL_MACHINE:
    HKEY = winreg.HKEY_LOCAL_MACHINE
    DRIVERS = Key(HKEY, r"SYSTEM\MountedDevices")
    ENVIRONMENT = Key(HKEY, r"SYSTEM\CurrentControlSet\Control\Session Manager\Environment")
    RUN = Key(HKEY, r"SOFTWARE\Microsoft\Windows\CurrentVersion\Run")


class HKEY_CURRENT_USER(Key):
    HKEY = winreg.HKEY_CURRENT_USER
    SHELL_FOLDER = Key(HKEY, r"SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders")
    RUN = Key(HKEY, r"SOFTWARE\Microsoft\Windows\CurrentVersion\Run")


def get_drives():
    """Get the list of available drives."""
    drives = []
    key = HKEY_LOCAL_MACHINE.DRIVERS.open()
    prefix = "\\DosDevices\\"
    index = 0
    try:
        while True:
            name = winreg.EnumValue(key, index)[0]
            if name.startswith(prefix):
                mount = name[len(prefix) :] + os.sep
                if os.path.exists(mount):
                    drives.append(mount)
            index += 1
    except WindowsError as exc:
        if exc.winerror != 259:
            raise exc
    return drives


def get_desktop_path():
    """Get desktop path."""
    key = HKEY_CURRENT_USER.SHELL_FOLDER.open()
    return winreg.QueryValueEx(key, "Desktop")[0]



"""Don't use this, is not accurate."""

__all__ = [
    # Angle
    "Angle",
    "Angle3",
    # Vector
    "Vector",
    "Vector3",
    # Coordination and entity
    "relative_pos",
    "relative_pos3",
    "Entity",
]

import math as _math


def relative_pos(coord, coord2):
    """相对二维坐标"""
    return coord2[0] - coord[0], coord2[1] - coord[1]


def relative_pos3(coord, coord2):
    """相对三维坐标"""
    return coord2[0] - coord[0], coord2[1] - coord[1], coord2[2] - coord[2]


class _Vec1:
    x = 0.0

    def __init__(self, x=0.0):
        if x:
            self.x = x

    def __repr__(self):
        return "%s(%s)" % (self.__class__.__name__, self.x)

    def __int__(self):
        return int(self.x)

    def __float__(self):
        return float(self.x)

    def __neg__(self):
        return self.__class__(-self.x)

    def __add__(self, other):
        if isinstance(other, self.__class__):
            return self.__class__(self.x + other.x)
        elif isinstance(other, (int, float)):
            return self.__class__(self.x + other)
        raise TypeError

    def __eq__(self, __o):
        if isinstance(__o, self.__class__):
            return self.x == __o.x
        return self.x == __o


class _Vec2:
    x = 0.0
    y = 0.0

    def __init__(self, x=0.0, y=0.0):
        if x:
            self.x = x
        if y:
            self.y = y

    def __repr__(self):
        return "%s(x=%s, y=%s)" % (self.__class__.__name__, self.x, self.y)

    def __len__(self):
        return 2

    def __tuple__(self):
        return self.x, self.y

    def __iter__(self):
        return iter((self.x, self.y))

    def __getitem__(self, key):
        if key == 0:
            return self.x
        if key == 1:
            return self.y
        raise KeyError

    def __neg__(self):
        return self.__class__(-self.x, -self.y)

    def __add__(self, other):
        if isinstance(other, self.__class__):
            return self.__class__(self.x + other.x, self.y + other.y)
        raise TypeError

    def __sub__(self, other):
        if isinstance(other, self.__class__):
            return self.__class__(self.x - other.x, self.y - other.y)
        raise TypeError

    def __mul__(self, other):
        if isinstance(other, self.__class__):
            return self.__class__(self.x * other.x, self.y * other.y)
        if isinstance(other, (int, float)):
            return self.__class__(self.x * other, self.y * other)
        raise TypeError

    def __iadd__(self, other):
        if isinstance(other, self.__class__):
            self.x += other.x
            self.y += other.y
            return self
        raise TypeError

    def __isub__(self, other):
        if isinstance(other, self.__class__):
            self.x -= other.x
            self.y -= other.y
            return self
        raise TypeError

    def __imul__(self, other):
        if isinstance(other, self.__class__):
            self.x *= other.x
            self.y *= other.y
            return self
        if isinstance(other, (int, float)):
            self.x *= other
            self.y *= other
            return self
        raise TypeError

    def __eq__(self, otehr):
        if isinstance(otehr, self.__class__):
            return self.x == otehr.x and self.y == otehr.y
        return (self.x, self.y) == otehr

    def add(self, other):
        return self + other

    def sub(self, other):
        return self - other

    def mul(self, other):
        return self * other


class _Vec3:
    x = 0.0
    y = 0.0
    z = 0.0

    def __init__(self, x=0.0, y=0.0, z=0.0):
        if x:
            self.x = x
        if y:
            self.y = y
        if z:
            self.z = z

    def __repr__(self):
        return "%s(x=%s, y=%s, z=%s)" % (self.__class__.__name__, self.x, self.y, self.z)

    def __len__(self):
        return 3

    def __tuple__(self):
        return self.x, self.y, self.z

    def __iter__(self):
        return iter((self.x, self.y, self.z))

    def __getitem__(self, key):
        if key == 0:
            return self.x
        if key == 1:
            return self.y
        if key == 2:
            return self.z
        raise KeyError

    def __neg__(self):
        return self.__class__(-self.x, -self.y, -self.z)

    def __add__(self, other):
        if isinstance(other, self.__class__):
            return self.__class__(self.x + other.x, self.y + other.y, self.z + other.z)
        raise TypeError

    def __sub__(self, other):
        if isinstance(other, self.__class__):
            return self.__class__(self.x - other.x, self.y - other.y, self.z - other.z)
        raise TypeError

    def __mul__(self, other):
        if isinstance(other, self.__class__):
            return self.__class__(self.x * other.x, self.y * other.y, self.z * other.z)
        if isinstance(other, (int, float)):
            return self.__class__(self.x * other, self.y * other, self.z * other)
        raise TypeError

    def __iadd__(self, other):
        if isinstance(other, self.__class__):
            self.x += other.x
            self.y += other.y
            self.z += other.z
            return self
        raise TypeError

    def __isub__(self, other):
        if isinstance(other, self.__class__):
            self.x -= other.x
            self.y -= other.y
            self.z -= other.z
            return self
        raise TypeError

    def __imul__(self, other):
        if isinstance(other, self.__class__):
            self.x *= other.x
            self.y *= other.y
            self.z *= other.z
            return self
        if isinstance(other, (int, float)):
            self.x *= other
            self.y *= other
            self.z *= other
            return self
        raise TypeError

    def __eq__(self, __o):
        if isinstance(__o, self.__class__):
            return self.x == __o.x and self.y == __o.y and self.z == __o.z
        return (self.x, self.y, self.z) == __o

    def add(self, other):
        return self + other

    def sub(self, other):
        return self - other

    def mul(self, other):
        return self * other


class Angle(_Vec1):
    _x = 0.0

    def vec(self, distance: float = 1):
        return Vector(
            _math.sin(self._x * _math.pi / 180) * distance,
            _math.cos(self._x * _math.pi / 180) * distance,
        )

    @property
    def x(self):
        return self._x

    @x.setter
    def x(self, value):
        value = value % 360
        if value >= 180:
            value = -180 + value % 180
        self._x = value


class Angle3(_Vec3):
    _x = 0.0
    _y = 0.0
    _z = 0.0

    def vec(self, distance: float = 1.0):
        if not distance:
            return Vector3()
        return Vector3(0, distance, 0).rotate(-self)

    @property
    def x(self):
        return self._x

    @x.setter
    def x(self, value):
        value = value % 360
        if value >= 180:
            value = -180 + value % 180
        self._x = value

    @property
    def y(self):
        return self._y

    @y.setter
    def y(self, value):
        value = value % 360
        if value >= 180:
            value = -180 + value % 180
        self._y = value

    @property
    def z(self):
        return self._z

    @z.setter
    def z(self, value):
        value = value % 360
        if value >= 180:
            value = -180 + value % 180
        self._z = value


class Vector(_Vec2):
    def rotate(self, angle):
        angle = -angle * _math.pi / 180
        return Vector(
            self.x * _math.cos(angle) - self.y * _math.sin(angle),
            self.y * _math.cos(angle) + self.x * _math.sin(angle),
        )

    def one(self):
        if self.y:
            angle = _math.atan(abs(self.x / self.y))
        else:
            angle = 90 * _math.pi / 180
        return Vector(
            _math.sin(angle) if self.x >= 0 else -_math.sin(angle),
            _math.cos(angle) if self.y >= 0 else -_math.cos(angle),
        )


class Vector3(_Vec3):
    def rotate(self, angle):
        x, y = Vector(self.x, self.y).rotate(angle[2])
        x, z = Vector(x, self.z).rotate(angle[1])
        z, y = Vector(z, y).rotate(angle[0])
        return Vector3(x, y, z)


class Entity:
    name = "Entity"
    enable = True
    visible = True
    ignore = False

    def __init__(self, **kwargs):
        self._pos = Vector3()
        self._rotation = Angle3()
        for k, v in kwargs.items():
            setattr(self, k, v)

    def __repr__(self):
        return "%s name=%s>" % (object.__repr__(self)[:-1], self.__class__.__name__)

    def update(self, dt):
        pass

    @property
    def pos(self):
        return self._pos

    @pos.setter
    def pos(self, value):
        self._pos = Vector3(*value)

    @property
    def rotation(self):
        return self._rotation

    @rotation.setter
    def rotation(self, value):
        self._rotation = Angle3(*value)



from __future__ import annotations

import os
import socket
import sys
from collections import defaultdict
from typing import Any

import httpx


def get_local_ip() -> str:
    """Get local IP address."""
    try:
        with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as sock:
            sock.connect(("8.8.8.8", 80))
            return sock.getsockname()[0]
    except socket.error:
        return socket.gethostbyname(socket.gethostname())


if sys.platform == "win32":

    def get_local_ip2() -> tuple[str, str]:
        """Get local IP address, return (ipv4, ipv6)."""
        output = [line for line in os.popen("ipconfig")]
        ipv4 = ""
        ipv6 = ""
        for i, line in enumerate(output):
            if "IPv4" in line and output[i + 2][-2] != " ":
                ipv4 = line[line.find(":") + 2 : -1]
            elif "IPv6" in line and output[i + 3][-2] != " ":
                ipv6 = line[line.find(":") + 2 : -1]
        return ipv4, ipv6


def get_public_ip() -> str:
    """Get public IP address.

    Return the public IP address if it is available, otherwise return 127.0.0.1.
    """
    try:
        return httpx.get("https://api.ipify.org").text
    except httpx.RequestError:
        return "127.0.0.1"


def get_public_ip_json() -> dict[str, Any]:
    """Get public IP address as JSON."""
    try:
        return httpx.get("https://ipinfo.io/json").json()
    except httpx.RequestError:
        return defaultdict(str, {"ip": "127.0.0.1"})


def get_ip_info(ip: str) -> dict[str, Any]:
    """Retrieves the information about the IP address."""
    url = f"https://freegeoip.app/json/{ip}"
    headers = {"accept": "application/json", "content-type": "application/json"}
    try:
        return httpx.get(url, headers=headers).json()
    except httpx.RequestError:
        return defaultdict(str, {"ip": "127.0.0.1"})



"""版本信息"""

from __future__ import annotations
import datetime


class VersionInfo:
    def __init__(
        self,
        major=0,
        minor=0,
        micro=0,
        build=0,
        release: str = "release",
        date: datetime.date | None = None,
    ):
        self.major = major
        self.minor = minor
        self.micro = micro
        self.build = build
        self.release = release
        self.date = date

    def __str__(self):
        return self.version

    @property
    def version(self):
        return f"{self.major}.{self.minor}.{self.micro}"

    @property
    def full_version(self):
        return f"{self.version}.{self.build}"

    @property
    def release_version(self):
        return f"{self.version} {self.release} {self.build}"


if __name__ == "__main__":
    version_info = VersionInfo(1, 1, 1, 0, "Release", datetime.date(2022, 3, 25))
    print(version_info.version)



from __future__ import annotations

import math
import multiprocessing
import os

from rich.console import Console
from rich.progress import (
    MofNCompleteColumn,
    Progress,
    SpinnerColumn,
    TimeElapsedColumn,
    TimeRemainingColumn,
)

from hmeqotools.mathutil import dec
from hmeqotools.mathutil import funcs as mfn

console = Console()


def _blend_list(lst, unit_size, start_index=0):
    """拌匀列表元素.

    :param lst: 列表
    :param unit_size: 多少元素视作一个单位，按照这个单位排序
    :param start_index: 起始排序位置
    """
    lst_len = len(lst)
    sort_len = lst_len - start_index
    if sort_len > 2:
        sort_len = lst_len - lst_len % unit_size
        mid_index = sort_len // 2
        mid_index = mid_index - mid_index % unit_size + start_index
        sort_len += start_index
        for i, i2 in zip(range(start_index, mid_index, unit_size * 2), range(sort_len, mid_index + 1, -unit_size * 2)):
            lst[i : i + unit_size], lst[i2 - unit_size : i2] = lst[i2 - unit_size : i2], lst[i : i + unit_size]


class Pi:
    k = 10

    @classmethod
    def prec(cls, precision=16) -> int:
        k = cls.k + len(str(precision)) + 1
        pre = precision + k
        b = 10**pre
        x1 = b * 4 // 5
        x2 = b // -239
        s1 = -25
        s2 = -57121
        result = x1 + x2

        n = int(pre * 1.5)
        for i in range(3, n, 2):
            x1 //= s1
            x2 //= s2
            x = (x1 + x2) // i
            if x == 0:
                break
            result += x
        result = result * 4 // 10**k
        return result

    @classmethod
    def ramanujan(cls, n: int, is_prec=True, space_for_time=False) -> int:
        """Ramanujan formulas.

        WARNING: is used multiprocessing,
        please write in `if __name__ == __main__`.
        """
        cpu_count = os.cpu_count()
        assert isinstance(cpu_count, int)
        with multiprocessing.Pool(processes=cpu_count) as pool:
            with Progress(
                SpinnerColumn(),
                *Progress.get_default_columns()[:-1],
                MofNCompleteColumn(),
                TimeElapsedColumn(),
                TimeRemainingColumn(),
                console=console,
                transient=False,
            ) as progress:
                task1 = progress.add_task("Ramanujan", total=None)
                task2 = progress.add_task("Ramanujan", total=1, start=False)
                valid_digits = 14
                if is_prec:
                    iter_count = n / valid_digits
                    if iter_count % 1:
                        iter_count += 1
                    iter_count = int(iter_count)
                else:
                    iter_count = n
                prec = valid_digits * iter_count
                progress.update(task1, total=iter_count)

                result = dec.Decimal(0, 0, prec + cls.k)
                count = 4 * cpu_count

                if space_for_time:
                    ftrs = [i for i in mfn.factorial_gen(6 * iter_count)]
                    params = [(i, result, (ftrs[i], ftrs[3 * i], ftrs[6 * i])) for i in range(iter_count)]
                    del ftrs
                    # 对参数排序，使进度条过渡更平滑
                    _blend_list(params, cpu_count)
                    while params:
                        result += sum(pool.map(cls._ramanujan2, params[:count]))
                        progress.update(task1, advance=len(params[:count]))
                        del params[:count]
                else:
                    params = [(i, result) for i in range(iter_count)]
                    _blend_list(params, cpu_count)
                    while params:
                        result += sum(pool.map(cls._ramanujan, params[:count]))
                        progress.update(task1, advance=len(params[:count]))
                        del params[:count]

                progress.start_task(task2)
                result = 426880 * result.convert_int(10005).root(2) / result
                result = result.n // 10 ** (result.dp - (n if is_prec else prec))
                progress.update(task2, completed=1)
        return result

    @staticmethod
    def _ramanujan(k):
        k, dec_ = k
        return dec_.convert_int(math.factorial(6 * k) * (13591409 + 545140134 * k)) / (
            math.factorial(3 * k) * math.factorial(k) ** 3 * (-640320) ** (3 * k)
        )

    @staticmethod
    def _ramanujan2(k):
        k, dec_, ftrs = k
        return dec_.convert_int(ftrs[2] * (13591409 + 545140134 * k)) / (ftrs[1] * ftrs[0] ** 3 * (-640320) ** (3 * k))

    @classmethod
    def machin(cls, precision=16) -> int:
        """马青公式 (内置进度条)."""
        with Progress(
            SpinnerColumn(),
            *Progress.get_default_columns()[:-1],
            MofNCompleteColumn(),
            TimeElapsedColumn(),
            TimeRemainingColumn(),
            console=console,
            transient=False,
        ) as progress:
            task1 = progress.add_task("Machin", total=None)
            console.log("初始化变量...")
            # 多计算k位，防止尾数取舍的影响
            k = cls.k + len(str(precision)) + 1
            pre = precision + k
            b = 10**pre
            # 求含4/5的首项
            # 求含1/239的首项
            x1 = b * 4 // 5
            x2 = b // -239
            # x1 = b * 16 // 5
            # x2 = b * 4 // -239
            s1 = -25
            s2 = -57121
            # 求第一大项
            result = x1 + x2

            # 设置下面循环的终点，即共计算n项
            n = int(2 * pre * 0.75)
            progress.update(task1, total=n + 2)
            # 循环初值=3，末值2n,步长=2
            for i in range(3, n, 2):
                # 求每个含1/5的项及符号
                x1 //= s1
                # 求每个含1/239的项及符号
                x2 //= s2
                # 求两项之和
                x = (x1 + x2) // i
                if x == 0:
                    break
                # 求总和
                result += x
                progress.update(task1, advance=2)
            # 求出π 并 舍掉后k位
            # result //= 10 ** k
            result = result * 4 // 10**k
            progress.update(task1, completed=n + 2)
        return result


class E:
    k = 10

    @classmethod
    def taylor(cls, precision=16) -> int:
        """Taylor formulas.

        !!! warning
            warning: is used multiprocessing
        """
        # 计算时的精度
        p1 = precision + len(str(precision)) + cls.k
        # 迭代次数，精度越高，此数值越小
        if p1 < 100:
            # p1 < 100
            p2 = p1
        elif p1 < 500:
            # 100 <= p1 < 500
            p2 = 65 * p1 // 100
        elif p1 < 1000:
            # 500 <= p1 < 1000
            p2 = 50 * p1 // 100
        elif p1 < 5000:
            # 1000 <= p1 < 5000
            p2 = 45 * p1 // 100
        elif p1 < 10000:
            # 5000 <= p1 < 10000
            p2 = 36 * p1 // 100
        elif p1 < 100000:
            # 10000 <= p1 < 100000
            p2 = 33 * p1 // 100
        else:
            # 100000 <= p1
            p2 = 30 * p1 // 100

        cpu_count = os.cpu_count()
        assert isinstance(cpu_count, int)
        with multiprocessing.Pool(processes=cpu_count) as pool:
            with Progress(
                SpinnerColumn(),
                *Progress.get_default_columns()[:-1],
                MofNCompleteColumn(),
                TimeElapsedColumn(),
                TimeRemainingColumn(),
                console=console,
                transient=False,
            ) as progress:
                task1 = progress.add_task("Taylor", total=None)
                result = dec.Decimal(0, 0, p1)
                one = result.convert_int(1)
                count = 64 * cpu_count

                params = [(one, i) for i in mfn.factorial_gen(p2)]
                _blend_list(params, count)
                progress.update(task1, total=len(params))
                while params:
                    result += sum(pool.map(cls._taylor, params[:count]))
                    progress.update(task1, advance=len(params[:count]))
                    del params[:count]
        return result.n // 10 ** (result.dp - precision)

    @staticmethod
    def _taylor(k):
        return k[0] / k[1]


def main():
    E.taylor(50000)


if __name__ == "__main__":
    main()



import threading
import tkinter as tk
import traceback


def on_error(exc: Exception):
    print(
        "--------------------------------------------------\n"
        + "".join(traceback.format_exception(type(exc), exc, exc.__traceback__))
        + "--------------------------------------------------"
    )


def percent_validate(variable, default="100.0%"):
    try:
        value = variable.get().rstrip("%")
        value = str(float(value)) + "%"
    except Exception:
        value = default
    variable.set(value.lstrip("-"))


def float_validate(variable, default="0.0"):
    try:
        value = variable.get()
        value = str(float(value))
    except Exception:
        value = default
    variable.set(value)


def int_validate(variable, default="0"):
    try:
        value = variable.get()
        value = str(int(value))
    except Exception:
        value = default
    variable.set(value)


def adapt_widgets_width(widgets, req=True):
    """将控件列表中的控件的宽度设置为其中最大的宽度"""
    width = max(i.winfo_reqwidth() if req else i.winfo_width() for i in widgets)
    for i in widgets:
        i.configure(width=width)


class Window:
    def __init__(self, size=None, root=None, start_quit_thr=True):
        # 窗口
        self.tk = root if root else tk.Tk()
        self.tk.wm_withdraw()
        self.tk.protocol("WM_DELETE_WINDOW", self.quit)
        self.resolution = (self.tk.winfo_screenwidth(), self.tk.winfo_screenheight())
        if size:
            self.tk.geometry("{}x{}".format(*size))
        # 控制窗口退出
        self.alive = True
        # 激活状态
        self.activated = threading.Event()
        # 显示中
        self.showed = threading.Event()
        # 控制窗口退出
        self.ctrl_quit = threading.Event()
        self.thr_quit = threading.Thread(target=self._quit_window, daemon=True)
        if start_quit_thr:
            self.thr_quit.start()

    def _quit_window(self):
        """离开窗口事件处理."""
        while self.ctrl_quit.wait() and self.alive:
            if self.activated.is_set():
                self.activated.clear()
                try:
                    self.on_quit()
                except Exception as exc:
                    on_error(exc)
                try:
                    self.tk.wm_withdraw()
                except RuntimeError as exc:
                    on_error(exc)
                self.tk.quit()
                self.ctrl_quit.clear()

    def mainloop(self, show=True):
        """启动窗口."""
        self.on_mainloop()
        self.activated.set()
        if show:
            try:
                self.tk.wm_deiconify()
                self.tk.focus_force()
            except RuntimeError as exc:
                on_error(exc)
        self.tk.mainloop()

    def quit(self):
        """离开窗口."""
        self.ctrl_quit.set()

    def destroy(self):
        """销毁窗口."""
        self.alive = False
        self.activated.set()
        self.ctrl_quit.set()
        self.tk.quit()
        self.tk.destroy()
        self.on_destroy()

    def hide(self):
        self.tk.wm_withdraw()
        self.showed.clear()

    def show(self):
        self.tk.wm_deiconify()
        self.showed.set()

    def iconify(self):
        self.tk.wm_iconify()

    def on_mainloop(self):
        pass

    def on_quit(self):
        pass

    def on_destroy(self):
        pass

    def winfo_size(self):
        return self.tk.winfo_width(), self.tk.winfo_height()

    def winfo_reqsize(self):
        return self.tk.winfo_reqwidth(), self.tk.winfo_reqheight()

    def winfo_xy(self):
        return self.tk.winfo_x(), self.tk.winfo_y()

    def bind(self, *args, **kwargs):
        self.tk.bind(*args, **kwargs)

    def align_center(self, req=False):
        self.tk.update()
        w, h = self.winfo_reqsize() if req else self.winfo_size()
        self.tk.wm_geometry("+{}+{}".format((self.resolution[0] - w) // 2, (self.resolution[1] - h) // 2))


class TkMethods:
    def __init__(self, tk_obj):
        self.mainloop = tk_obj.mainloop
        self.quit = tk_obj.quit
        self.destroy = tk_obj.destroy


class WidgetMethods:
    def __init__(self, widget):
        self.pack = widget.pack
        self.place = widget.place
        self.grid = widget.grid
        self.pack_forget = widget.pack_forget
        self.place_forget = widget.place_forget
        self.grid_forget = widget.grid_forget
        self.configure = widget.configure


class MouseMessage:
    """鼠标事件绑定"""

    def __init__(self, widget):
        self.mouse_status_over = False
        self.mouse_status_left = False
        self.mouse_status_middle = False
        self.mouse_status_right = False
        self.mouse_event = None
        widget.bind("<Enter>", self.__on_mouse_enter)
        widget.bind("<Leave>", self.__on_mouse_leave)
        widget.bind("<Button-1>", self.__on_mouse_left_down)
        widget.bind("<ButtonRelease-1>", self.__on_mouse_left_up)
        widget.bind("<Button-2>", self.__on_mouse_middle_down)
        widget.bind("<ButtonRelease-2>", self.__on_mouse_middle_up)
        widget.bind("<Button-3>", self.__on_mouse_right_down)
        widget.bind("<ButtonRelease-3>", self.__on_mouse_right_up)
        widget.bind("<Motion>", self.__on_mouse_motion)
        widget.bind("<B1-Motion>", self.__on_mouse_left_motion)
        widget.bind("<B2-Motion>", self.__on_mouse_middle_motion)
        widget.bind("<B3-Motion>", self.__on_mouse_right_motion)
        widget.bind("<Double-Button-1>", self.__on_mouse_left_dclick)
        widget.bind("<Double-Button-2>", self.__on_mouse_middle_dclick)
        widget.bind("<Double-Button-3>", self.__on_mouse_right_dclick)
        widget.bind("<MouseWheel>", self.__on_mouse_wheel)

    def __on_mouse_enter(self, event):
        self.mouse_status_over = True
        self.mouse_event = event
        self.on_mouse_enter()

    def __on_mouse_leave(self, event):
        self.mouse_status_over = False
        self.mouse_event = event
        self.on_mouse_leave()

    def __on_mouse_left_down(self, event):
        self.mouse_status_left = True
        self.mouse_event = event
        self.on_mouse_left_down()

    def __on_mouse_left_up(self, event):
        self.mouse_status_left = False
        self.mouse_event = event
        self.on_mouse_left_up()

    def __on_mouse_middle_down(self, event):
        self.mouse_status_middle = True
        self.mouse_event = event
        self.on_mouse_middle_down()

    def __on_mouse_middle_up(self, event):
        self.mouse_status_middle = False
        self.mouse_event = event
        self.on_mouse_middle_up()

    def __on_mouse_right_down(self, event):
        self.mouse_status_right = True
        self.mouse_event = event
        self.on_mouse_right_down()

    def __on_mouse_right_up(self, event):
        self.mouse_status_right = False
        self.mouse_event = event
        self.on_mouse_right_up()

    def __on_mouse_motion(self, event):
        self.mouse_event = event
        self.on_mouse_motion()

    def __on_mouse_left_motion(self, event):
        self.mouse_event = event
        self.on_mouse_left_motion()

    def __on_mouse_middle_motion(self, event):
        self.mouse_event = event
        self.on_mouse_middle_motion()

    def __on_mouse_right_motion(self, event):
        self.mouse_event = event
        self.on_mouse_right_motion()

    def __on_mouse_left_dclick(self, event):
        self.mouse_event = event
        self.on_mouse_left_dclick()

    def __on_mouse_middle_dclick(self, event):
        self.mouse_event = event
        self.on_mouse_middle_dclick()

    def __on_mouse_right_dclick(self, event):
        self.mouse_event = event
        self.on_mouse_right_dclick()

    def __on_mouse_wheel(self, event):
        self.mouse_event = event
        self.on_mouse_wheel()

    def on_mouse_enter(self):
        pass

    def on_mouse_leave(self):
        pass

    def on_mouse_left_down(self):
        pass

    def on_mouse_left_up(self):
        pass

    def on_mouse_middle_down(self):
        pass

    def on_mouse_middle_up(self):
        pass

    def on_mouse_right_down(self):
        pass

    def on_mouse_right_up(self):
        pass

    def on_mouse_motion(self):
        pass

    def on_mouse_left_motion(self):
        pass

    def on_mouse_middle_motion(self):
        pass

    def on_mouse_right_motion(self):
        pass

    def on_mouse_left_dclick(self):
        pass

    def on_mouse_middle_dclick(self):
        pass

    def on_mouse_right_dclick(self):
        pass

    def on_mouse_wheel(self):
        pass


class KeyboardMessage:
    """键盘事件绑定"""

    def __init__(self, widget: tk.Misc):
        self._held_key = {}
        self.keyboard_event = None
        widget.bind("<KeyPress>", self.__on_key_press)
        widget.bind("<KeyRelease>", self.__on_key_release)

    def __on_key_press(self, event):
        self._held_key[event.keysym] = event
        self.keyboard_event = event
        self.on_key_press()

    def __on_key_release(self, event):
        if event.keysym in self._held_key:
            del self._held_key[event.keysym]
        self.keyboard_event = event
        self.on_key_release()

    def held_keys(self):
        return self._held_key.copy()

    def get_key(self, name=None):
        return self._held_key.get(name)

    def clear_held_key(self):
        self._held_key.clear()

    def on_key_press(self):
        pass

    def on_key_release(self):
        pass


class MsgBox(tk.Listbox):
    def __init__(self, *args, **kwargs):
        kwa = {"height": 1}
        kwa.update(kwargs)
        super().__init__(*args, **kwa)
        self._count = 0
        self.max = 32

    def append(self, *args):
        if self.max and self._count > self.max:
            self.delete(0)
        else:
            self._count += 1
        self.insert("end", *args)
        self.yview("moveto", 1.0)

    def clear(self):
        self.delete(0, "end")


class Var:
    def __init__(self, value):
        self.value = value
        self.bind = {}

    def apply_changes(self):
        for func in self.bind.values():
            func(self.value)
        self.on_changes()

    def set(self, value):
        self.value = value
        self.apply_changes()

    def get(self):
        return self.value

    def on_changes(self):
        pass


class IntVar(Var):
    def __init__(self, value=0):
        super().__init__(value)


class FloatVar(Var):
    def __init__(self, value=0.0):
        super().__init__(value)


class StringVar(Var):
    def __init__(self, value=""):
        super().__init__(value)


class BooleanVar(Var):
    def __init__(self, value=False):
        super().__init__(value)


class Option(WidgetMethods, MouseMessage):
    def __init__(self, master, text="", value=None, variable=None, size=(100, 32), **kwargs):
        self.master = master
        self.selected = False
        self.fg = "#000000"
        self.fg_mouse_enter = self.fg
        self.fg_mouse_held = self.fg
        self.fg_select = "#ffffff"
        self.bg = "#ffffff"
        self.bg_mouse_enter = "#99ccff"
        self.bg_mouse_held = "#66aaff"
        self.bg_select = "#3388dd"
        self.value = value
        self.variable = None
        self.size = size

        self.f = tk.Frame(self.master, width=self.size[0], height=self.size[1])
        super().__init__(self.f)
        self.f.pack_propagate(False)
        self.label = tk.Label(self.f, text=text, bg=self.bg, fg=self.fg)
        self.label.pack(fill="both", expand=True)
        MouseMessage.__init__(self, self.label)
        if variable:
            self.set_variable(variable)

        for k, v in kwargs.items():
            setattr(self, k, v)

    def resize(self, size):
        self.f.configure(width=size[0], height=size[1])
        self.size = size

    def set_variable(self, variable: Var):
        self.variable = variable
        self.variable.bind[self.value] = self.apply_select_change

    def set_text(self, text):
        self.label.configure(text=text)

    def configure_label(self, **kwargs):
        self.label.configure(kwargs)

    def on_mouse_enter(self):
        # 鼠标进入
        if not self.mouse_status_left and not self.selected:
            self.label.configure(fg=self.fg_mouse_enter, bg=self.bg_mouse_enter)

    def on_mouse_leave(self):
        # 鼠标离开
        if not self.mouse_status_left and not self.selected:
            self.label.configure(fg=self.fg, bg=self.bg)

    def on_mouse_left_down(self):
        # 鼠标按下
        if not self.selected:
            self.label.configure(fg=self.fg_mouse_held, bg=self.bg_mouse_held)

    def on_mouse_left_up(self):
        # 鼠标在范围内松开即选中
        if self.variable and not self.selected and self.mouse_status_over:
            self.variable.set(self.value)

    def apply_select_change(self, value):
        if value == self.value:
            if not self.selected:
                self._select()
        elif self.selected:
            self._unselect()

    def _select(self):
        self.label.configure(fg=self.fg_select, bg=self.bg_select)
        self.selected = True
        self.on_select()

    def _unselect(self):
        self.label.configure(fg=self.fg, bg=self.bg)
        self.selected = False
        self.on_unselect()

    def init(self):
        """初始化, 由其他对象调用此方法."""
        pass

    def on_select(self):
        """选中."""
        pass

    def on_unselect(self):
        """取消选中."""
        pass


class OptionGroup(WidgetMethods):
    def __init__(self, master, variable: Var, size=(100, 32), **kwargs):
        self.master = master
        self.size = size
        self.variable = variable

        self.f = tk.Frame(self.master)
        super().__init__(self.f)
        self.options: list[Option] = []

        for k, v in kwargs.items():
            setattr(self, k, v)

    def configure_options(self, index=None, **kwargs):
        if index:
            self.options[index].configure_label(**kwargs)
        else:
            for option in self.options:
                option.configure_label(**kwargs)

    def init(self):
        for i in self.options:
            i.init()

    def add(self, value=None, text=None, option=None, **kwargs):
        if isinstance(option, Option):
            op = option
            if text is not None:
                op.set_text(text)
            if value is not None:
                op.value = value
        else:
            if option is None:
                option = Option
            op = option(self.f, text=text if text else "", value=value, **kwargs)
        op.set_variable(self.variable)
        op.resize(self.size)
        op.pack()
        self.options.append(op)

    def switch(self, value):
        self.variable.set(value)


def main():
    import tkinter.messagebox

    class MyOption(Option):
        def __init__(self, w, name, *args, **kwargs):
            self.w = w
            self.name = name
            super().__init__(*args, **kwargs)

    class MyWindow(Window, KeyboardMessage):
        """Inherit Window."""

        def __init__(self):
            super().__init__(size=(800, 500))
            self.f = tk.Frame(self.tk)
            self.f.pack(fill="both", expand=True)
            KeyboardMessage.__init__(self, self.tk)
            self.ops = OptionGroup(self.f, Var("1"), size=(100, 50))
            self.ops.pack()
            self.ops.add(option=MyOption(self, "1", self.f, "1", "1"))
            self.ops.add(option=MyOption(self, "2", self.f, "2", "2"))
            self.ops.add(option=MyOption(self, "3", self.f, "3", "3"))
            self.ops.configure_options(font=("Consolas", 12, "bold"))
            self.ops.switch("1")
            self.mb = MsgBox(self.f)
            self.mb.pack(side="bottom", fill="x")
            self.mb.append("123")
            self.align_center()

        def quit(self):
            if tkinter.messagebox.askyesno("Exit", "Are you sure you want to exit?"):
                super().quit()

        def on_mainloop(self):
            print("Window is mainloop.")

        def on_quit(self):
            print("Quiting window.")

        def on_destroy(self):
            print("Window is destroyed.")

        def on_key_press(self, event):
            print("<down>:", event.keysym, self.held_keys())

        def on_key_release(self, event):
            print("<up>:", event.keysym, self.held_keys())

    window = MyWindow()
    window.mainloop()
    window.destroy()


if __name__ == "__main__":
    main()



from __future__ import annotations

import decimal
import hashlib
import random
from typing import Union

from hmeqotools import mathutil
from hmeqotools.mathutil import predict


def subsection_str(obj, k, right=False):
    """分段字符串."""
    if not obj:
        return None
    if right:
        index = divmod(len(obj), k)[1]
        if index:
            yield obj[:index]
            obj = obj[index:]
    while obj:
        yield obj[:k]
        obj = obj[k:]


class RSA:
    """RSA加密算法."""

    def __init__(self, n: int, e: int, d: int):
        self._n_len = 1
        self._n = 1
        self.n = n
        self.e = e
        self.d = d
        self.encoding = "UTF-8"

    @classmethod
    def generate_key(cls, digits=1024, e=None, pqk=2):
        """随机生成n、公钥、私钥.
        可指定公钥, 例如e=65537.
        返回n、公钥、私钥.
        digits 最好不小于 24.
        """
        while True:
            # 生成多个素数并计算fy_n和fy的值
            n, fy = cls.generate_n_fyn(digits, pqk)
            # 如果没有给出公钥则随机获取公钥
            if e is None:
                e = cls.generate_e(n, digits)
                break
            # 如果指定的e和n互质，退出循环，否则重新获取n, fy
            elif mathutil.gcd(n, e) == 1:
                break
        # 得到私钥
        d = cls.generate_d(fy, e)
        # 返回n, 公钥, 私钥
        return n, e, d

    @classmethod
    def generate_n_fyn(cls, digits: int, k=2):
        """随机生成n和fy."""
        n = fy = 1
        # 产生k个素数并计算n和fy
        for i in cls.random_prime(digits, k):
            n *= i
            fy *= i - 1
        return n, fy

    @classmethod
    def generate_e(cls, n, digits: int):
        """根据n随机创建公钥."""
        while True:
            # 随机生成素数并判断是否和n互质
            e = cls.random_prime(digits)[0]
            if mathutil.gcd(n, e) == 1:
                break
        return e

    @staticmethod
    def generate_d(fy, e):
        """根据公钥和fy获取一个私钥."""
        # 扩展欧里几得算法快速获取d
        d = mathutil.gcd_ext_euclid(fy, e)[1]
        if d < 0:
            d += fy
        return d

    @staticmethod
    def generate_d_list(fy, e, k=1, start=0):
        """生成一个或多个私钥.
        返回含d的列表, 和k值.
        """
        results = []
        i = start
        while len(results) < k:
            i += 1
            d = (i * fy + 1) / e
            d_int = int(d)
            if d == d_int and d_int not in results:
                results.append(d_int)
        return results, i

    @staticmethod
    def random_prime(digits, pq_count=1, k=None):
        """随机获取素数.

        Args:
            digits (int): 二进制位数
            pq_count (int, optional): 分成几份, 例如获取p, q, 值取2. Defaults to 1.
            k (int, optional): 获取几个素数, 默认为 pq_count. Defaults to None.
        """
        pos = 1
        if pq_count != 1:
            # 获取多个素数并合成
            decimal.getcontext().prec = predict.predict_digits(digits)
            digits = decimal.Decimal(str(digits)) / pq_count
            # 填补分多个素数合成带来的误差
            pos = 2 ** (decimal.Decimal(str(pq_count)) - 1) / pq_count
        # 素数范围
        range_b = 2**digits
        range_a = range_b // 2 * pos
        range_a, range_b = int(range_a), int(range_b - 1)

        results = []
        for _ in range(pq_count if k is None else k):
            while True:
                # 随机数
                result = random.randint(range_a, range_b)
                if mathutil.is_prime(result) and result not in results:
                    results.append(result)
                    break
        return results

    def encrypt_key(self, password: Union[str, bytes], digits: int | None = None):
        """加密 RSA key"""
        if digits is None:
            digits = int(len(str(self.n)) / 2.56)
        if isinstance(password, str):
            password = password.encode("ASCII")
        pwd_code = int(hashlib.shake_128(password).hexdigest(digits), 16)
        return self.n ^ pwd_code, self.e ^ pwd_code, self.d ^ pwd_code

    def encrypt(self, integer, e=None):
        """加密整数"""
        return pow(integer, self.e if e is None else e, self.n)

    def decrypt(self, integer, d=None):
        """解密整数"""
        return pow(integer, self.d if d is None else d, self.n)

    def encrypt_bytes(self, obj: bytes, e=None, hexadecimal=True):
        """加密字节"""
        s = str(int(obj.hex(), 16))
        # 分段加密字节
        result = ""
        for i in subsection_str(s, self._n_len - 1, right=True):
            result += str(self.encrypt(int(i), e)).zfill(self._n_len)
        # 转换十六进制
        if hexadecimal:
            result = hex(int(result))[2:]
        # 转换为字节
        result = result.encode("ASCII")
        return result

    def decrypt_bytes(self, obj: bytes, d=None, hexadecimal=True):
        """解密字节"""
        # 解析16进制
        s = str(int(obj, 16)) if hexadecimal else obj
        # 分段解密字符串
        result = ""
        for i in subsection_str(s, self._n_len, right=True):
            result += str(self.decrypt(int(i), d)).zfill(self._n_len - 1)
        # 还原字节
        result = hex(int(result))[2:]
        result = bytes.fromhex(result)
        return result

    def encrypt_str(self, obj: str, e=None, hexadecimal=True):
        """加密字符串"""
        # 转换为数字串
        obj = str(int(obj.encode(self.encoding).hex(), 16))
        # 分段加密字符串
        result = ""
        for i in subsection_str(obj, self._n_len - 1, right=True):
            result += str(self.encrypt(int(i), e)).zfill(self._n_len)
        # 转换十六进制
        if hexadecimal:
            result = hex(int(result))[2:]
        return result

    def decrypt_str(self, obj: str, d=None, hexadecimal=True):
        """解密字符串"""
        # 解析16进制
        if hexadecimal:
            obj = str(int(obj, 16))
        # 分段解密字符串
        result = ""
        for i in subsection_str(obj, self._n_len, right=True):
            result += str(self.decrypt(int(i), d)).zfill(self._n_len - 1)
        # 还原
        result = hex(int(result))[2:]
        result = bytes.fromhex(result).decode(self.encoding)
        return result

    def get_key(self):
        """获取n, e, d"""
        return self.n, self.e, self.d

    @property
    def n(self):
        return self._n

    @n.setter
    def n(self, value: int):
        self._n = value
        self._n_len = len(str(value))


def main():
    # n, e, d
    rsa = RSA(n=3189370259, e=3961537159, d=2672103319)
    print(rsa.get_key())
    # 原文
    text = "Hello World"
    # 密文
    encrypt = rsa.encrypt_str(text)
    print("密文:\t", encrypt)

    # 明文
    decrypt = rsa.decrypt_str(encrypt)
    print("明文:\t", decrypt)


if __name__ == "__main__":
    main()



from __future__ import annotations

import random

from hmeqotools import mathutil


class DH:
    def __init__(self, p: int, g: int, e: int):
        """Diffie-Hellman算法

        Arguments:
            p (int): 质数
            g (int): 随机整数
            e (int): 私钥
        """
        self.p = p
        self.g = g
        self.e = e

    @staticmethod
    def generate_p(bits: int):
        """随机质数"""
        range_b = 2**bits
        range_a = range_b >> 1
        range_b -= 1
        while True:
            num = random.randint(range_a, range_b)
            if mathutil.is_prime(num):
                return num

    @staticmethod
    def generate_g(p: int):
        """根据p的大小生成随机整数"""
        return random.randint(p >> 1, p - 1)

    @classmethod
    def generate(cls, p: int | None = None, bits=2048):
        p = cls.generate_p(bits)
        g = cls.generate_g(p)
        return DH(p, g, p if p else cls.randint(100000, 999999))

    def public_key(self):
        """合成公钥"""
        return pow(self.g, self.e, self.p)

    def final(self, key: int):
        return pow(key, self.e, self.p)

    randint = random.randint


def main():
    dh = DH.generate()
    # 生成公钥
    public_key = dh.public_key()
    print(public_key)
    # 发送给对方，并接受对方的公钥
    other_public_key = int(input("请输入对方的公钥: "))
    # 合成共同密钥
    aes_key = dh.final(other_public_key)
    print(aes_key)


if __name__ == "__main__":
    main()



from __future__ import annotations

import base64
import os

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes

from hmeqotools.crypto.padding import *


class AES(Cipher):
    modes = modes

    block_size = 16

    def __init__(self, key: bytes, mode: modes.Mode, block_size=block_size):
        super().__init__(algorithms.AES(key), mode, default_backend())
        self.key = key
        self.block_size = block_size

    @classmethod
    def generate_key(cls):
        return os.urandom(cls.block_size)

    @classmethod
    def generate(cls, mode: modes.Mode | None = None):
        return cls(cls.generate_key(), mode or modes.ECB())

    @classmethod
    def pad(cls, b: bytes):
        """padding bytes"""
        return pad(b, cls.block_size)

    @staticmethod
    def unpad(b: bytes):
        """unpadding bytes"""
        return unpad(b)

    def encrypt(self, data: bytes):
        """Encrypt bytes, automatic padding data"""
        encryptor = self.encryptor()
        encrypted = encryptor.update(self.pad(data)) + encryptor.finalize()
        return base64.b64encode(encrypted)

    def decrypt(self, data: bytes):
        """Decrypt bytes, automatic unpadding data"""
        a = base64.b64decode(data)
        decryptor = self.decryptor()
        decrypted = decryptor.update(a) + decryptor.finalize()
        return self.unpad(decrypted)


def main():
    key = b"1234567890123456"
    # key = AES.generate_key()
    aes = AES(key, AES.modes.ECB())
    a = aes.encrypt(b"123")
    b = aes.decrypt(a)
    print(a, b)


if __name__ == "__main__":
    main()



"""排序相关算法"""

from .search import binary_search


def bubble_sort(lst):
    """冒泡排序"""
    keys = list(lst)
    for i in range(len(keys) - 1, 0, -1):
        for j in range(0, i):
            k = j + 1
            if keys[j] > keys[k]:
                keys[j], keys[k] = keys[k], keys[j]
    return keys


def selection_sort(lst):
    """选择排序"""
    keys = list(lst)
    for i in range(len(keys)):
        for j in range(i + 1, len(keys)):
            if keys[i] > keys[j]:
                keys[i], keys[j] = keys[j], keys[i]
    return keys


def insertion_sort(lst):
    """插入排序"""
    keys = list(lst)
    for i in range(0, len(keys) - 1):
        j = i + 1
        while keys[i] > keys[j]:
            keys[i], keys[j] = keys[j], keys[i]
            if not i:
                break
            i, j = i - 1, i
    return keys


def quick_sort(lst, left=0, right=None, key=None):
    """快速排序
    非递归 元素交换实现
    """
    if right is None:
        right = len(lst) - 1
    orient = True
    pointer_list = [(left, right)]
    lst = list(lst)
    keys = [key(i) for i in lst] if key else lst
    key = key is not None
    while pointer_list:
        pl, pr = left, right = pointer_list.pop()
        while pl < pr:
            if keys[pl] > keys[pr]:
                keys[pl], keys[pr] = keys[pr], keys[pl]
                if key:
                    lst[pl], lst[pr] = lst[pr], lst[pl]
                orient = not orient
            if orient:
                pr -= 1
            else:
                pl += 1
        pl, pr = pl - 1, pr + 1
        if left < pl:
            pointer_list.append((left, pl))
        if pr < right:
            pointer_list.append((pr, right))
    return lst


def quick_sort2(lst, left=0, right=None):
    """快速排序
    递归实现
    """
    if right is None:
        right = len(lst) - 1
    if right - left > 1:
        border_left, border_right = left, right
        pivot = lst[left]
        while left < right:
            while left < right and lst[right] >= pivot:
                right -= 1
            lst[left] = lst[right]
            while left < right and lst[left] <= pivot:
                left += 1
            lst[right] = lst[left]
        lst[left] = pivot
        quick_sort2(lst, border_left, left - 1)
        quick_sort2(lst, right + 1, border_right)


def quick_sort3(lst, left=0, right=None):
    """快速排序
    非递归元素覆盖实现
    """
    if right is None:
        right = len(lst) - 1
    pointer_list = [(left, right)]
    while pointer_list:
        left, right = border_left, border_right = pointer_list.pop()
        pivot = lst[left]
        while left < right:
            while left < right and lst[right] >= pivot:
                right -= 1
            lst[left] = lst[right]
            while left < right and lst[left] <= pivot:
                left += 1
            lst[right] = lst[left]
        lst[left] = pivot
        left, right = left - 1, right + 1
        if border_left < left:
            pointer_list.append((border_left, left))
        if right < border_right:
            pointer_list.append((right, border_right))


def radix_sort(lst):
    """基数排序."""
    lst = list(lst)
    buckets = [[] for _ in range(10)]
    b1 = 10
    b2 = 1
    for _ in range(len(str(max(lst)))):
        for i in lst:
            buckets[(i % b1 - i % b2) // b2].append(i)
        lst = [j for i in buckets for j in i]
        for i in buckets:
            i.clear()
        b1, b2 = b1 * 10, b1
    return lst


class Sort:
    """插入排序，从小到大."""

    def __init__(self, lst=(), key=None):
        self.data = sorted(lst, key=key) if lst else []
        self.key = key

    def __repr__(self):
        return "%s(%r)" % (self.__class__.__name__, self.data)

    def __str__(self):
        return "%s(%s)" % (self.__class__.__name__, self.data)

    def __bool__(self):
        return bool(self.data)

    def __len__(self):
        return len(self.data)

    def __iter__(self):
        return iter(self.data)

    def __getitem__(self, item):
        return self.data[item]

    def __setitem__(self, key, value):
        self.data[key] = value

    def __delitem__(self, key):
        del self.data[key]

    def __reversed__(self):
        return reversed(self)

    def __contains__(self, item):
        return item in self.data

    def get_list(self):
        """获取列表."""
        return self.data

    def length(self):
        """长度."""
        return len(self.data)

    def find(self, value, key=None):
        """查找元素."""
        if key:
            for index, i in enumerate(self.data):
                if key(i) == value:
                    return index
            return None
        return binary_search(self.data, value, key=self.key)

    def add(self, value):
        """添加."""
        index = binary_search(self.data, self.key(value) if self.key else value, key=self.key, insertion=True)
        self.data.insert(index, value)
        return index

    def remove(self, value):
        try:
            self.data.remove(value)
        finally:
            pass

    def delete(self, start, stop=None):
        """删除."""
        if stop is None:
            del self.data[start]
        else:
            del self.data[start:stop]

    def clear(self):
        """清空."""
        self.data.clear()



from __future__ import annotations

import hashlib
import inspect
import re
import types as _types

DEFAULT_SIZE = 1000


def get_frame_vc(frame: _types.FrameType, size=DEFAULT_SIZE) -> int:
    """Get the vc of frame."""
    return get_vc(inspect.getsource(frame), size)


def get_vc(s: str, size=DEFAULT_SIZE) -> int:
    """Get the vc of str."""
    return int(hashlib.sha256(s.encode()).hexdigest(), 16) % size


def force_vc(frame: inspect._SourceObjectType, size=DEFAULT_SIZE) -> int | None:
    """Force the vc of frame."""
    source = inspect.getsource(frame)
    match = re.search(r"(?<=%s\()\d+" % inspectcodechange.__name__, source)
    if match:
        start, end = match.span()
        code_p1, code_p2 = source[:start], source[end:]
        for i in range(size):
            new_code = code_p1 + str(i) + code_p2
            vc = get_vc(new_code)
            if vc == i:
                return i
    return None


def inspectcodechange(vc: int) -> bool:
    """Inspect current frame code change."""
    frame = inspect.getouterframes(inspect.currentframe())[1].frame
    frame_code = inspect.getsource(frame)
    return vc == get_vc(frame_code)


def main(n=10):
    if inspectcodechange(7):
        print("验证成功")
    else:
        print("验证失败, 我的代码被修改了")
        return 0  # 整个bug
    return 2**n


if __name__ == "__main__":
    print(f"vc: {force_vc(main)}")
    print(main())



from __future__ import annotations

import re
from decimal import Decimal
from operator import add, mod, mul, pow, sub, truediv
from typing import Callable


class Operator:
    def __init__(self, op: Convertor, count=2, priority=1):
        self.op = op
        self.count = count
        self.priority = priority

    def __str__(self) -> str:
        return f"{self.__class__.__name__}({self.op.__name__}, priority={self.priority})"

    def calc(self, *args: Decimal):
        if len(args) != self.count:
            raise ValueError
        return self.op(*args)

    __repr__ = __str__


class Bracket:
    def __init__(self, pair: str | None = None, is_head=False):
        self.pair = pair
        self.is_head = is_head

    def __str__(self) -> str:
        return f"{self.__class__.__name__}(pair={repr(self.pair)})"

    __repr__ = __str__


class Symbol:
    Operator = Operator
    Bracket = Bracket


Convertor = Callable[..., Decimal]
symbols: dict[re.Pattern, Convertor | Operator | Bracket] = {
    re.compile(r"(?<!\d)(\+|-)?\d+(\.\d+)?"): Decimal,
    re.compile(r"\+"): Operator(add),
    re.compile(r"-"): Operator(sub),
    re.compile(r"\*"): Operator(mul, priority=2),
    re.compile(r"x"): Operator(mul, priority=2),
    re.compile(r"/"): Operator(truediv, priority=2),
    re.compile(r"÷"): Operator(truediv, priority=2),
    re.compile(r"%"): Operator(mod, priority=2),
    re.compile(r"mod"): Operator(mod, priority=2),
    re.compile(r"\^"): Operator(pow, priority=3),
    re.compile(r"\*\*"): Operator(pow, priority=3),
    re.compile(r"\("): Bracket("()", is_head=True),
    re.compile(r"\)"): Bracket("()"),
    re.compile(r"\["): Bracket("[]", is_head=True),
    re.compile(r"\]"): Bracket("[]"),
}


def parse_symbol(expression: str, pos: int):
    """解析表达式并返回解析后的结果"""
    for pattern, op in sorted(
        symbols.items(), key=lambda x: x[1].priority if isinstance(x[1], Operator) else -1, reverse=True
    ):
        span = pattern.match(expression, pos=pos)
        if span is None:
            continue
        return op, span
    raise ValueError("不支持的符号 %r" % expression[pos])


def calc(expression: str) -> Decimal:
    return calc_preprocessed(preprocess(expression))


def preprocess(expression: str) -> list[Decimal | Operator]:
    stack: list[Operator | Bracket] = []
    preprocessed: list[Decimal | Operator] = []

    expression = expression.replace(" ", "")
    pos = 0
    while pos < len(expression):
        symbol, span = parse_symbol(expression, pos)
        if isinstance(symbol, Operator):
            while stack and isinstance(stack[-1], Operator) and stack[-1].priority >= symbol.priority:
                preprocessed.append(stack[-1])
                del stack[-1]
            stack.append(symbol)
        elif isinstance(symbol, Bracket):
            if symbol.is_head:
                stack.append(symbol)
            else:
                while isinstance(stack[-1], Operator):
                    preprocessed.append(stack[-1])
                    del stack[-1]
                if isinstance(stack[-1], Bracket) and stack[-1].pair != symbol.pair:
                    raise ValueError("不匹配的括号 %r" % symbol.pair)
                del stack[-1]
        else:
            preprocessed.append(symbol(expression[span.start() : span.end()]))
        pos = span.end()

    preprocessed.extend(reversed([i for i in stack if isinstance(i, Operator)]))
    return preprocessed


def calc_preprocessed(preprocessed: list[Decimal | Operator]):
    stack: list[Decimal] = []
    for sign in preprocessed:
        if isinstance(sign, Operator):
            if len(stack) < sign.count:
                raise ValueError("表达式错误")
            result = sign.calc(*stack[-sign.count :])
            stack = stack[: -sign.count]
            stack.append(result)
        else:
            stack.append(sign)

    if len(stack) != 1:
        raise ValueError("表达式错误")
    return stack[0]


def main():
    result = calc("1 + (6 + 2) * 3")
    print(result)


if __name__ == "__main__":
    main()



from typing import Union, overload


@overload
def sequential_search(lst, item, start=0, stop=None, key=None) -> Union[int, None]: ...


@overload
def sequential_search(lst, item, start=0, stop=None, key=None, insertion=True) -> int: ...


@overload
def binary_search(lst, item, low=0, high=None, key=None) -> Union[int, None]: ...


@overload
def binary_search(lst, item, low=0, high=None, key=None, insertion=True) -> int: ...


def sequential_search(lst, item, start=0, stop=None, key=None, insertion=False):
    """顺序查找"""
    found = False
    if stop is None:
        stop = len(lst)
    for index in range(start, stop):
        value = lst[index]
        if key:
            value = key(value)
        if item <= value:
            if item == value:
                found = True
            break
    else:
        index = len(lst)
    return index if found or insertion else None


def binary_search(lst, item, low=0, high=None, key=None, insertion=False):
    """二分查找"""
    found = False
    mid = low
    if high is None:
        high = len(lst) - 1
    while low <= high:
        mid = (low + high) // 2
        value = lst[mid]
        if key:
            value = key(value)
        if item > value:
            low = mid = mid + 1
        elif item < value:
            high = mid - 1
        else:
            found = True
            break
    return mid if found or insertion else None
